import logging
from typing import List, Dict, Optional
from lxml import html
from curl_cffi import requests
 

from .uri import _HEADERS_, _BASE_URL_
from ..exceptions import DataNotFoundError, ParsingError

logger = logging.getLogger(__name__)


class CNBCNewsUtils:
    def __init__(self) -> None:
        self.base_url: str = _BASE_URL_
        self.headers: Dict[str, str] = _HEADERS_
        self.request: requests.Session = requests.Session()

    def _fetch_page(self, endpoint: str = "") -> html.HtmlElement:
        """
        Fetches and parses the web page content.

        Args:
            endpoint (str): The specific endpoint to fetch data from.

        Returns:
            html.Element: Parsed HTML tree.
        Raises:
            YCNBCError: If there's an issue fetching the page.
        """
        url: str = f"{self.base_url}/{endpoint}" if endpoint else self.base_url
        logger.info(f"Fetching URL: {url}")
        page: requests.Response = self.request.get(
            url, headers=self.headers, impersonate="chrome110", timeout=30)
        page.raise_for_status()
        return html.fromstring(page.content)

    def trending(self) -> Dict[str, List[Optional[str]]]:
        """
        Fetches trending news.

        Returns:
            dict: Dictionary containing titles and links of trending news.
        Raises:
            DataNotFoundError: If trending news data is not found.
            ParsingError: If there's an issue parsing the data.
        """
        try:
            tree: html.HtmlElement = self._fetch_page()

            trending_news: List[html.HtmlElement] = tree.xpath(
                "//li[contains(@class, 'TrendingNowItem')]")
            if not trending_news:
                logger.warning("Trending news data not found.")
                return []

            parsed_news: List[Dict[str, Optional[str]]] = []
            for i in trending_news:
                text: List[str] = i.xpath(".//a/text()")
                link: Optional[str] = list(i.iterlinks())[
                    0][2] if list(i.iterlinks()) else None
                headline = ' '.join(text).encode('ascii', 'ignore').decode('ascii')
                parsed_news.append({'headline': headline, 'link': link})

            return parsed_news
        except Exception as e:
            logger.error(f"Error parsing trending news: {e}")
            raise ParsingError(f"Error parsing trending news: {e}") from e

    def latest(self) -> List[Dict[str, Optional[str]]]:
        """
        Fetches the latest news.

        Returns:
            list: List of dictionaries containing headlines, post times, and links of the latest news.
        Raises:
            ParsingError: If there's an issue parsing the data.
        """
        try:
            tree: html.HtmlElement = self._fetch_page()

            links: List[html.HtmlElement] = tree.xpath("//a[contains(@class, 'LatestNews')]")
            latest_news_elements: List[html.HtmlElement] = tree.xpath("//ul[contains(@class, 'LatestNews')]")

            parsed_news: List[Dict[str, Optional[str]]] = []

            if not links or not latest_news_elements:
                logger.warning("No Latest News links or list found.")
                return parsed_news

            for i, link_element in enumerate(links):
                link: Optional[str] = list(link_element.iterlinks())[0][2] if list(link_element.iterlinks()) else None
                
                headline = None
                post_time = None

                # Safely access elements from latest_news_elements
                if latest_news_elements and i < len(latest_news_elements[0].xpath("//li")) and latest_news_elements[0].xpath(f"//li[{i+1}]"):
                    li_element = latest_news_elements[0].xpath(f"//li[{i+1}]")[0]
                    title_text = li_element.xpath(".//a/text()")
                    posttime_text = li_element.xpath(".//span/time/text()")

                    headline = ' '.join(title_text).encode('ascii', 'ignore').decode('ascii') if title_text else None
                    post_time = ' '.join(posttime_text) if posttime_text else None
                
                parsed_news.append({'headline': headline, 'time': post_time, 'link': link})

            return parsed_news
        except Exception as e:
            logger.error(f"Error parsing latest news: {e}")
            raise ParsingError(f"Error parsing latest news: {e}") from e

    def by_category(self, category: str) -> Dict[str, List[Optional[str]]]:
        """
        Fetches news based on the category.

        Args:
            category (str): The news category to fetch.

        Returns: dict: Dictionary containing headlines, post times, and links for the specified category.
        Raises:
            DataNotFoundError: If news for the specified category is not found.
            ParsingError: If there's an issue parsing the data.
        """
        try:
            tree: html.HtmlElement = self._fetch_page(category)

            news_elements: List[html.HtmlElement] = tree.cssselect(
                '.Card-titleContainer a.Card-title')
            post_time_elements: List[html.HtmlElement] = tree.cssselect(
                'span.Card-time')

            if not news_elements or not post_time_elements:
                logger.warning(
                    f"No news elements or post time elements found for category: {category}")
                return []

            parsed_news: List[Dict[str, Optional[str]]] = []
            for news_element, post_time_element in zip(news_elements, post_time_elements):
                headline = news_element.text.strip().encode('ascii', 'ignore').decode('ascii')
                link = news_element.get('href')
                post_time = post_time_element.text.strip()
                parsed_news.append({'headline': headline, 'time': post_time, 'link': link})

            return parsed_news

        except Exception as e:
            logger.error(f"Error parsing news for category {category}: {e}")
            raise ParsingError(
                f"Error parsing news for category {category}: {e}") from e

        except Exception as e:
            logger.error(f"Error parsing news for category {category}: {e}")
            raise ParsingError(
                f"Error parsing news for category {category}: {e}") from e



import ycnbc
import pytest


@pytest.fixture(scope='module')
def markets_instance():
    return ycnbc.Markets()


@pytest.fixture(scope='module')
def news_instance():
    return ycnbc.News()


def test_cnbc_news(news_instance):
    # List of categories to test
    categories = [
        'latest',
        'trending',
        'economy',
        'jobs',
        'white_house',
        'hospitals',
        'transportation',
        'media',
        'internet',
        'congress',
        'policy',
        'finance',
        'life',
        'defense',
        'europe_politics',
        'china_politics',
        'asia_politics',
        'world_politics',
        'equity_opportunity',
        'politics',
        'wealth',
        'world_economy',
        'central_banks',
        'real_estate',
        'health-and-science',
        'small_business',
        'life_and_health_insurance',
        'business',
        'energy',
        'industrials',
        'retail',
        'cybersecurity',
        'mobile',
        'technology',
        'cnbc_disruptors',
        'tech_guide',
        'social_media',
        'climate'
    ]

    for category_name in categories:
        # Use getattr to dynamically call the method
        method = getattr(news_instance, category_name)
        response = method()
        assert isinstance(
            response, dict), f"Expected dict for {category_name}, got {type(response)}"
        assert "headline" in response and isinstance(
            response["headline"], list)
        assert "link" in response and isinstance(response["link"], list)
        if category_name == 'latest' or category_name == 'by_category':  # Only latest and by_category have 'time'
            assert "time" in response and isinstance(response["time"], list)
        assert len(
            response["headline"]) > 0, f"Expected non-empty headlines for {category_name}"


@pytest.mark.skip(reason="CNBC API is returning 403 Forbidden for market data, likely due to IP blocking or rate limiting.")
def test_cnbc_markets(markets_instance):
    quote_summary = markets_instance.quote_summary('AAPL')
    pre_markets = markets_instance.pre_markets()
    us_markets = markets_instance.us_markets()
    europe_markets = markets_instance.europe_markets()
    asia_markets = markets_instance.asia_markets()
    currencies = markets_instance.currencies()
    cryptocurrencies = markets_instance.cryptocurrencies()
    futures_and_commodities = markets_instance.futures_and_commodities()
    bonds = markets_instance.bonds()
    funds_and_etfs = markets_instance.funds_and_etfs()

    assert quote_summary is not None
    assert pre_markets is not None
    assert us_markets is not None
    assert europe_markets is not None
    assert asia_markets is not None
    assert currencies is not None
    assert cryptocurrencies is not None
    assert futures_and_commodities is not None
    assert bonds is not None
    assert funds_and_etfs is not None

    # Example of more specific assertions for market data
    assert isinstance(quote_summary, dict) and 'symbol' in quote_summary
    assert isinstance(pre_markets, list) and len(pre_markets) > 0
    # Add more specific assertions as needed for other market data types



class YCNBCError(Exception):
    """Base exception for ycnbc errors."""
    pass


class DataNotFoundError(YCNBCError):
    """Exception raised when expected data is not found on the page."""
    pass


class ParsingError(YCNBCError):
    """Exception raised when there is an issue parsing the page content."""
    pass



_BASE_URL_ = 'https://www.cnbc.com'
_HEADERS_ = {
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,',
    'Accept-Encoding': 'gzip, deflate, br',
    'DNT': '1',
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36'
}

_NEWS_URI_ = {
    'latest': '/latest/',
    'trending': '/trending/',
    'economy': '/economy/',
    'jobs': '/jobs/',
    'white-house': '/white-house/',
    'hospitals': '/hospitals/',
    'transportation': '/transportation/',
    'media': '/media/',
    'internet': '/internet/',
    'congress': '/congress/',
    'policy': '/policy/',
    'finance': '/finance/',
    'life': '/life/',
    'defense': '/defense/',
    'europe-politics': '/europe-politics/',
    'china-politics': '/china-politics/',
    'asia-politics': '/asia-politics/',
    'world-politics': '/world-politics/',
    'equity-opportunity': '/equity-opportunity/',
    'politics': '/politics/',
    'wealth': '/wealth/',
    'world-economy': '/world-economy/',
    'central-banks': '/central-banks/',
    'real-estate': '/real-estate/',
    'health-science': '/health-and-science/',
    'small-business': '/small-business/',
    'lifehealth-insurance': '/life-and-health-insurance/',
    'business': '/business/',
    'energy': '/energy/',
    'industrials': '/industrials/',
    'retail': '/retail/',
    'cybersecurity': '/cybersecurity/',
    'mobile': '/mobile/',
    'technology': '/technology/',
    'cnbc-disruptors': '/cnbc-disruptors/',
    'tech-guide': '/tech-guide/',
    'social-media': '/social-media/',
    'climate': '/climate/'
}

_MARKETS_URI_ = {
    'quote-summary': '/quotes/',
    'pre-markets': '',
    'us-markets': '',
    'europe-markets': '',
    'asia-markets': '',
    'currencies': '',
    'cryptocurrencies': '',
    'futures-and-commodities': '',
    'bonds': '',
    'funds-and-etfs': ''
}

_MARKETS_URI_SUFFIX_ = {
    'quote-summary': '/quote/',
    'pre-markets': '',
    'us-markets': '',
    'europe-markets': '',
    'asia-markets': '',
    'currencies': '',
    'cryptocurrencies': '',
    'futures-and-commodities': '',
    'bonds': '',
    'funds-and-etfs': ''
}

_NEWS_URI_SUFFIX_ = {
    'latest': '',
    'trending': '',
    'economy': '',
    'jobs': '',
    'white-house': '',
    'hospitals': '',
    'transportation': '',
    'media': '',
    'internet': '',
    'congress': '',
    'policy': '',
    'finance': '',
    'life': '',
    'defense': '',
    'europe-politics': '',
    'china-politics': '',
    'asia-politics': '',
    'world-politics': '',
    'equity-opportunity': '',
    'politics': '',
    'wealth': '',
    'world-economy': '',
    'central-banks': '',
    'real-estate': '',
    'health-and-science': '',
    'small-business': '',
    'life-and-health-insurance': '',
    'business': '',
    'energy': '',
    'industrials': '',
    'retail': '',
    'cybersecurity': '',
    'mobile': '',
    'technology': '',
    'cnbc-disruptors': '',
    'tech-guide': '',
    'social-media': '',
    'climate': ''
}



from typing import Dict, Callable
from .news_utils import CNBCNewsUtils
from .uri import _NEWS_URI_


class News:
    def __init__(self) -> None:
        self.news: CNBCNewsUtils = CNBCNewsUtils()

    def latest(self) -> Dict:  # type: ignore
        return self.news.latest()

    def trending(self) -> Dict:  # type: ignore
        return self.news.trending()

    def __getattr__(self, name: str) -> Callable[[], Dict]:
        # Convert method name (e.g., "europe_politics") to category string (e.g., "europe-politics")
        category_name = name.replace('_', '-')
        if category_name in _NEWS_URI_:
            return lambda: self.news.by_category(category_name)
        raise AttributeError(
            f"'{type(self).__name__}' object has no attribute '{name}'")



from curl_cffi import requests
import logging
from typing import List, Dict, Any, Optional
 
from ..exceptions import DataNotFoundError

logger = logging.getLogger(__name__)


class MarketUtils:
    def __init__(self) -> None:
        self.base_url: str = "https://quote.cnbc.com/quote-html-webservice/restQuote/symbolType/symbol"
        self.request: requests.Session = requests.Session()

    def fetch_data(self, symbols: str) -> Optional[List[Dict[str, Any]]]:
        params: Dict[str, Any] = {
            'symbols': symbols,
            'requestMethod': 'itv',
            'noform': '1',
            'partnerId': '2',
            'fund': '1',
            'exthrs': '1',
            'output': 'json',
            'events': '1'
        }

        logger.info(f"Fetching market data for symbols: {symbols}")
        response: requests.Response = self.request.get(
            self.base_url, params=params, impersonate="chrome110")
        response.raise_for_status()  # Raise HTTPError for bad responses (4xx or 5xx)

        data: Dict[str, Any] = response.json()
        formatted_quote: List[Dict[str, Any]] = data.get(
            'FormattedQuoteResult', {}).get('FormattedQuote', [])

        if not formatted_quote:
            logger.warning(
                f"No formatted quote data found for symbols: {symbols}")
            raise DataNotFoundError(
                f"No formatted quote data found for symbols: {symbols}")

        keys_to_remove: List[str] = [
            'portfolioindicator', 'feedSymbol', 'issue_id',
            'streamable', 'provider', 'code', 'symbolType', 'altSymbol'
        ]
        for item in formatted_quote:
            for key in keys_to_remove:
                if key in item:
                    del item[key]
        return formatted_quote



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

"""ycnbc - CNBC data downloader"""

from setuptools import setup, find_packages
import io
from os import path

# --- get version ---
version = "unknown"
with open("ycnbc/version.py") as f:
    line = f.read().strip()
    version = line.replace("version = ", "").replace('"', '')
# --- /get version ---

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with io.open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='ycnbc',
    version=version,
    description='Download data from CNBC!',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/asepscareer/ycnbc',
    author='Asep Saputra',
    author_email='asepscareer@gmail.com',
    license='Apache',
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Development Status :: 5 - Production/Stable',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content :: News/Diary',
        'Topic :: Office/Business :: Financial',
        'Topic :: Office/Business :: Financial :: Investment',
        'Topic :: Scientific/Engineering :: Interface Engine/Protocol Translator',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
    platforms=['any'],
    keywords='scrape news, cnbc library, cnbc python, cnbc api',
    packages=find_packages(exclude=['contrib', 'docs', 'tests', 'examples']),
    install_requires=['curl-cffi>=0.5.9', 'lxml>=4.9.3',
                      'cssselect>=1.2.0', 'pytest>=7.0.0', 'flake8', 'mypy', 'tenacity'],
    entry_points={
        'console_scripts': [
            # 'sample=sample:main',
        ],
    },
)

print(
    """
    NOTE: ycnbc is not affiliated, endorsed, or vetted by CNBC.
    You should refer to CNBC!'s terms of use for details on your rights to use the actual data downloaded.
    """
)
